@@ -1,19 +1,19 @@
 #![allow(dead_code)]
 use criterion::*;
 use tract_data::internal::*;
-use tract_linalg::frame::mmm::MatrixStore;
+use tract_linalg::frame::mmm::{FusedSpec, InputStore, PackedStore};
 use tract_linalg::frame::MatMatMul;
 
 use DatumType::*;
 
-pub fn packed_packed(c: &mut Criterion, name: &str, m: usize, k: usize, n: usize, tr: bool) {
+pub fn packed_packed(c: &mut Criterion, name: &str, m: usize, k: usize, n: usize) {
     let mut group = c.benchmark_group(format!("{}/packed_packed", name));
     group.throughput(Throughput::Elements((m * k * n) as u64));
     let id = format!("{}x{}x{}", m, k, n);
-    group.bench_with_input(BenchmarkId::new("f32/cold", &id), &(F32, m, k, n, true, tr), mat_mat);
-    group.bench_with_input(BenchmarkId::new("f32/hot", &id), &(F32, m, k, n, false, tr), mat_mat);
-    group.bench_with_input(BenchmarkId::new("i8/cold", &id), &(I8, m, k, n, true, tr), mat_mat);
-    group.bench_with_input(BenchmarkId::new("i8/hot", &id), &(I8, m, k, n, false, tr), mat_mat);
+    group.bench_with_input(BenchmarkId::new("f32/cold", &id), &(F32, m, k, n, true), mat_mat);
+    group.bench_with_input(BenchmarkId::new("f32/hot", &id), &(F32, m, k, n, false), mat_mat);
+    group.bench_with_input(BenchmarkId::new("i8/cold", &id), &(I8, m, k, n, true), mat_mat);
+    group.bench_with_input(BenchmarkId::new("i8/hot", &id), &(I8, m, k, n, false), mat_mat);
 }
 
 pub fn packed_vec(c: &mut Criterion, name: &str, m: usize, k: usize, n: usize) {
@@ -57,9 +57,8 @@ unsafe fn run(
     n: usize,
     be: &mut Bencher,
     mm: &dyn MatMatMul,
-    pa: &MatrixStore,
-    pb: &MatrixStore,
-    c: &mut MatrixStore,
+    pa: PackedStore,
+    pb: InputStore,
     cold: bool,
 ) {
     let mut scratch = mm.allocate_scratch_space();
@@ -70,7 +69,13 @@ unsafe fn run(
                 ruin_cache();
             }
             let instant = std::time::Instant::now();
-            mm.run_with_scratch_space(m, k, n, scratch.as_mut(), &pa, &pb, c, &[]).unwrap();
+            mm.run_with_scratch_space(
+                m,
+                n,
+                scratch.as_mut(),
+                &[FusedSpec::AddMatMul { a: pa, b: pb.clone(), k }],
+            )
+            .unwrap();
             let time = instant.elapsed();
             dur += time;
         }
@@ -78,25 +83,19 @@ unsafe fn run(
     });
 }
 
-fn mat_mat(
-    be: &mut Bencher,
-    &(dt, m, k, n, cold, c_trans): &(DatumType, usize, usize, usize, bool, bool),
-) {
+fn mat_mat(be: &mut Bencher, &(dt, m, k, n, cold): &(DatumType, usize, usize, usize, bool)) {
     let mm = tract_linalg::ops().mmm(dt, dt, dt, Some(m), Some(k), Some(n)).unwrap();
     let pa = Tensor::zero_aligned_dt(dt, &[mm.a_pack(k).len(m)], mm.a_pack(k).alignment()).unwrap();
     let pb = Tensor::zero_aligned_dt(dt, &[mm.b_pack(k).len(n)], mm.b_pack(k).alignment()).unwrap();
-    let mut c = Tensor::zero_dt(dt, &if c_trans { [n, m] } else { [m, n] }).unwrap();
     unsafe {
-        let c_sto = if c_trans { mm.c_view_with_axis(1, 0) } else { mm.c_view() };
         run(
             m,
             k,
             n,
             be,
             &*mm,
-            &mm.a_packed(dt.size_of(), k).wrap(&pa.view()),
-            &mm.b_packed(dt.size_of(), k).wrap(&pb.view()),
-            &mut c_sto.wrap(&mut c.view_mut()),
+            mm.a_packed(dt.size_of(), k).wrap(&pa.view()),
+            mm.b_packed(dt.size_of(), k).wrap(&pb.view()),
             cold,
         );
     }
@@ -107,7 +106,6 @@ fn mat_vec(be: &mut Bencher, &(dt, m, k, n, cold): &(DatumType, usize, usize, us
     let mm = tract_linalg::ops().mmm(dt, dt, dt, Some(m), Some(k), Some(n)).unwrap();
     let pa = Tensor::zero_aligned_dt(dt, &[mm.a_pack(k).len(m)], mm.a_pack(k).alignment()).unwrap();
     let pb = Tensor::zero_dt(dt, &[k, 1]).unwrap();
-    let mut c = Tensor::zero_dt(dt, &[m, n]).unwrap();
     unsafe {
         run(
             m,
@@ -115,9 +113,8 @@ fn mat_vec(be: &mut Bencher, &(dt, m, k, n, cold): &(DatumType, usize, usize, us
             n,
             be,
             &*mm,
-            &mm.a_packed(dt.size_of(), k).wrap(&pa.view()),
-            &mm.b_packed(dt.size_of(), k).wrap(&pb.view()),
-            &mut mm.c_view().wrap(&mut c.view_mut()),
+            mm.a_packed(dt.size_of(), k).wrap(&pa.view()),
+            mm.b_packed(dt.size_of(), k).wrap(&pb.view()),
             cold,
         );
     }
@@ -148,13 +145,21 @@ fn direct_conv_mmm_f32(be: &mut Bencher, geo: &ConvGeo) {
         be.iter(move || {
             mm.run(
                 m,
-                k,
                 n,
-                &mm.a_packed(f32::datum_type().size_of(), k).wrap(&pa.view()),
-                &mm.b_from_data_and_offsets(c.datum_type().size_of(), &rows_offsets, &cols_offsets)
-                    .wrap(&pb.view()),
-                &mut mm.c_view().wrap(&c.view_mut()),
-                &[],
+                &[
+                    FusedSpec::AddMatMul {
+                        a: mm.a_packed(f32::datum_type().size_of(), k).wrap(&pa.view()),
+                        b: mm
+                            .b_from_data_and_offsets(
+                                c.datum_type().size_of(),
+                                &rows_offsets,
+                                &cols_offsets,
+                            )
+                            .wrap(&pb.view()),
+                        k,
+                    },
+                    FusedSpec::Store(mm.c_view().wrap(&c.view_mut())),
+                ],
             )
         })
     }
@@ -172,17 +177,21 @@ fn direct_conv_i8(be: &mut Bencher, geo: &ConvGeo) {
         be.iter(move || {
             mm.run(
                 m,
-                k,
                 n,
-                &mm.a_packed(i8::datum_type().size_of(), k).wrap(&pa.view()),
-                &mm.b_from_data_and_offsets(
-                    pb.datum_type().size_of(),
-                    &rows_offsets,
-                    &cols_offsets,
-                )
-                .wrap(&pb.view()),
-                &mut mm.c_view().wrap(&c.view_mut()),
-                &[],
+                &[
+                    FusedSpec::AddMatMul {
+                        a: mm.a_packed(i8::datum_type().size_of(), k).wrap(&pa.view()),
+                        b: mm
+                            .b_from_data_and_offsets(
+                                pb.datum_type().size_of(),
+                                &rows_offsets,
+                                &cols_offsets,
+                            )
+                            .wrap(&pb.view()),
+                        k,
+                    },
+                    FusedSpec::Store(mm.c_view().wrap(&c.view_mut())),
+                ],
             )
         })
     }
