@@ -50,12 +50,8 @@ pub fn plug(ops: &mut Ops) {
         });
     }
 
-    ops.qmmm_i8_i8 = Box::new(|_, _, _| Box::new(MatMatMulImpl::<MatMatMulI8x8x8, i32>::new()));
-    ops.qmmv_i8_i8 = Box::new(|_, _| Box::new(MatMatMulImpl::<MatMatMulI8x64x1, i32>::new()));
-    ops.qmmm_i8_i32 =
-        Box::new(|_, _, _| Box::new(MatMatMulImpl::<MatMatMulI8xI32x8x8, i32>::new()));
-    ops.qmmv_i8_i32 = Box::new(|_, _| Box::new(MatMatMulImpl::<MatMatMulI8xI32x64x1, i32>::new()));
-
+    ops.qmmm_i32 = Box::new(|_, _, _| Box::new(MatMatMulImpl::<MatMatMulI8x8x8, i32>::new()));
+    ops.qmmv_i32 = Box::new(|_, _| Box::new(MatMatMulImpl::<MatMatMulI8x64x1, i32>::new()));
     ops.sigmoid_f32 = Box::new(|| Box::new(ElementWiseImpl::<SigmoidF32x4n, f32>::new()));
     ops.tanh_f32 = Box::new(|| Box::new(ElementWiseImpl::<TanhF32x4n, f32>::new()));
 }
@@ -72,8 +68,8 @@ fn best_of(
         let ker = kernels
             .iter()
             .min_by_key(|ker| {
-                let rows = m.div_ceil(ker.mr());
-                let cols = n.div_ceil(ker.nr());
+                let rows = m.divceil(ker.mr());
+                let cols = n.divceil(ker.nr());
                 let tiles = rows * cols;
                 let cost = if a53 {
                     match (ker.kernel_name(), ker.mr(), ker.nr()) {
