@@ -1,45 +1,22 @@
 use std::fmt::Debug;
 
-use super::*;
-
-#[repr(C)]
-#[derive(PartialEq, Copy, Clone, Debug)]
-pub struct MatMatMulKerSpec<'a, TI>
-where
-    TI: Copy + Debug,
-{
-    pub a: &'a PanelStore,
-    pub b: &'a PanelStore,
-    pub c: &'a Tile,
-    pub linear: &'a LinearSpec,
-    pub non_linear: *const FusedKerSpec<TI>,
-}
-
-#[repr(C, usize)]
-#[derive(PartialEq, Copy, Clone, Debug)]
-pub enum LinearSpec {
-    Mul { k: usize },
-    Noop,
-}
-
-impl LinearSpec {
-    pub fn k(k: usize) -> LinearSpec {
-        LinearSpec::Mul { k }
-    }
-}
+use crate::frame::mmm::FusedKerSpec;
 
 pub trait MatMatMulKer<TI>: Copy + Clone + Debug + Send + Sync + 'static
 where
     TI: Copy + Debug,
 {
     fn name() -> &'static str;
-    fn kernel(op: &MatMatMulKerSpec<TI>) -> isize;
+    fn kernel(op: &[FusedKerSpec<TI>]) -> isize;
     fn mr() -> usize;
     fn nr() -> usize;
     fn alignment_bytes_packed_a() -> usize;
     fn end_padding_packed_a() -> usize;
     fn alignment_bytes_packed_b() -> usize;
     fn end_padding_packed_b() -> usize;
+
+    #[allow(unused_variables)]
+    fn prefetch(ptr: *const u8, len: usize) {}
 }
 
 #[macro_export]
@@ -97,23 +74,11 @@ macro_rules! test_mmm_kernel_i8_u8_i32 {
     };
 }
 
-#[macro_export]
-macro_rules! test_mmm_kernel_u8 {
-    ($k: ty, $id: ident, $cond: expr) => {
-        #[cfg(test)]
-        #[allow(non_snake_case)]
-        mod $id {
-            mmm_kernel_tests!($cond, $k, u8, u8, u8, i32);
-            mmm_kernel_fuse_tests!($cond, $k, u8, i32);
-            qmmm_kernel_fuse_tests!($cond, $k, u8, u8, u8, i32);
-        }
-    };
-}
-
 #[cfg(test)]
 #[macro_use]
 pub mod test {
     use super::*;
+    use crate::frame::mmm::{InputStoreKer, OutputStoreKer, PackedStoreKer};
     use num_traits::{AsPrimitive, One, Zero};
     use proptest::collection::vec;
     use proptest::prelude::*;
@@ -203,10 +168,10 @@ pub mod test {
                         let len = pa.len() - 1;
                         pa[len] = 1 as _;
                         let pb = PackedOffsetsProblem::<$ker, $ta, $tb, $tc, $ti>::new(pa,
-                            vec!(0 as _, 1 as _),
-                            vec!(0usize; <$ker>::nr()),
-                            vec!(1usize, 0, 0),
-                            true);
+                                                                                       vec!(0 as _, 1 as _),
+                                                                                       vec!(0usize; <$ker>::nr()),
+                                                                                       vec!(1usize, 0, 0),
+                                                                                       true);
                         assert_eq!(pb.run(), pb.reference())
                     }
                 }
@@ -366,21 +331,27 @@ pub mod test {
                     .collect::<Vec<_>>();
                 let pb = Tensor::from_slice_align(&b, K::alignment_bytes_packed_b()).unwrap();
                 let mut v = vec![TC::zero(); K::mr() * K::nr()];
-                let mut c = if self.trans_c {
+                let c = if self.trans_c {
                     mmm_stride_storage(&mut v, 1, K::mr())
                 } else {
                     mmm_stride_storage(&mut v, K::nr(), 1)
                 };
-                let non_linear_ops = [FusedKerSpec::ScalarAdd(TI::one()), FusedKerSpec::Done];
-                let non_linear =
-                    if self.add_one { non_linear_ops.as_ptr() } else { std::ptr::null() };
-                let err = K::kernel(&MatMatMulKerSpec {
-                    a: &PanelStore::Packed { ptr: pa.as_ptr_unchecked::<TA>() as _ },
-                    b: &PanelStore::Packed { ptr: pb.as_ptr_unchecked::<TB>() as _ },
-                    c: &mut c,
-                    linear: &LinearSpec::k(self.k),
-                    non_linear,
+                let b_store =
+                    InputStoreKer::Packed(PackedStoreKer { ptr: pb.as_ptr_unchecked::<TB>() as _ });
+
+                let mut non_linear_ops = tvec!(FusedKerSpec::AddMatMul {
+                    k: self.k,
+                    pa: pa.as_ptr_unchecked::<u8>() as _,
+                    pb: &b_store,
+                    cpu_variant: 0,
                 });
+                if self.add_one {
+                    non_linear_ops.push(FusedKerSpec::ScalarAdd(TI::one()));
+                }
+                non_linear_ops.push(FusedKerSpec::Store(c));
+                non_linear_ops.push(FusedKerSpec::Done);
+                non_linear_ops.insert(0, FusedKerSpec::Clear);
+                let err = K::kernel(&non_linear_ops);
                 assert_eq!(err, 0);
                 v
             }
@@ -491,19 +462,27 @@ pub mod test {
                 self.cols_offsets.iter().map(|o| self.b.as_ptr().offset(*o as isize)).collect()
             };
             let mut v = vec![TC::zero(); K::mr() * K::nr()];
-            let mut c = mmm_stride_storage(&mut v, K::nr(), 1);
-            let non_linear_ops = [FusedKerSpec::ScalarAdd(TI::one()), FusedKerSpec::Done];
-            let non_linear = if self.add_one { non_linear_ops.as_ptr() } else { std::ptr::null() };
-            let err = K::kernel(&MatMatMulKerSpec {
-                a: &PanelStore::Packed { ptr: unsafe { pa.as_ptr_unchecked::<TA>() as _ } },
-                b: &PanelStore::OffsetsAndPtrs {
-                    row_byte_offsets: rows_offset.as_ptr(),
-                    col_ptrs: col_ptrs.as_ptr() as _,
-                },
-                c: &mut c,
-                linear: &LinearSpec::k(self.rows_offsets.len()),
-                non_linear,
-            });
+            let c = mmm_stride_storage(&mut v, K::nr(), 1);
+            let b_store = InputStoreKer::OffsetsAndPtrs {
+                row_byte_offsets: rows_offset.as_ptr(),
+                col_ptrs: col_ptrs.as_ptr() as _,
+            };
+
+            let mut non_linear_ops = tvec!(
+                FusedKerSpec::Clear,
+                FusedKerSpec::AddMatMul {
+                    k: self.rows_offsets.len(),
+                    pa: unsafe { pa.as_ptr_unchecked::<u8>() as _ },
+                    pb: &b_store,
+                    cpu_variant: 0,
+                }
+            );
+            if self.add_one {
+                non_linear_ops.push(FusedKerSpec::ScalarAdd(TI::one()));
+            }
+            non_linear_ops.push(FusedKerSpec::Store(c));
+            non_linear_ops.push(FusedKerSpec::Done);
+            let err = K::kernel(&non_linear_ops);
             assert_eq!(err, 0);
             v
         }
@@ -532,8 +511,8 @@ pub mod test {
         assert_eq!(pb.run(), pb.reference())
     }
 
-    pub fn mmm_stride_storage<T: Copy>(v: &mut [T], rsc: usize, csc: usize) -> Tile {
-        Tile {
+    pub fn mmm_stride_storage<T: Copy>(v: &mut [T], rsc: usize, csc: usize) -> OutputStoreKer {
+        OutputStoreKer {
             ptr: v.as_mut_ptr() as _,
             row_byte_stride: (std::mem::size_of::<T>() * rsc) as isize,
             col_byte_stride: (std::mem::size_of::<T>() * csc) as isize,
@@ -555,20 +534,27 @@ pub mod test {
         let b: Vec<TB> = (0..(k * t)).map(|x| x.as_()).collect();
         let len = K::mr() * K::nr();
         let mut v: Vec<TC> = vec![TC::zero(); len];
-        let mut c = mmm_stride_storage(&mut v, K::nr(), 1);
+        let c = mmm_stride_storage(&mut v, K::nr(), 1);
         let col_ptrs = (0..K::nr()).map(|i| (&b[i]) as *const TB as _).collect::<Vec<_>>();
         let row_byte_offsets =
             (0..k).map(|i| (i * std::mem::size_of::<TB>() * t) as isize).collect::<Vec<_>>();
-        let err = K::kernel(&MatMatMulKerSpec {
-            a: &PanelStore::Packed { ptr: unsafe { pa.as_ptr_unchecked::<TA>() as _ } },
-            b: &PanelStore::OffsetsAndPtrs {
-                col_ptrs: col_ptrs.as_ptr(),
-                row_byte_offsets: row_byte_offsets.as_ptr(),
+        let b_store = InputStoreKer::OffsetsAndPtrs {
+            row_byte_offsets: row_byte_offsets.as_ptr(),
+            col_ptrs: col_ptrs.as_ptr(),
+        };
+
+        let non_linear_ops = tvec!(
+            FusedKerSpec::Clear,
+            FusedKerSpec::AddMatMul {
+                k,
+                pa: unsafe { pa.as_ptr_unchecked::<u8>() as _ },
+                pb: &b_store,
+                cpu_variant: 0,
             },
-            c: &mut c,
-            linear: &LinearSpec::k(k),
-            non_linear: std::ptr::null(),
-        });
+            FusedKerSpec::Store(c),
+            FusedKerSpec::Done
+        );
+        let err = K::kernel(&non_linear_ops);
         assert_eq!(err, 0);
         let expected: Vec<TC> = (0..v.len())
             .map(|ix| {
@@ -602,19 +588,21 @@ pub mod test {
             .unwrap()
         };
         let b = vec![TB::one(); (k + 1) * K::nr()];
-        let c: Vec<TC> = vec![TC::zero(); K::mr() * K::nr()];
-        let err = K::kernel(&MatMatMulKerSpec {
-            a: &PanelStore::Packed { ptr: unsafe { pa.as_ptr_unchecked::<TA>() as _ } },
-            b: &PanelStore::Packed { ptr: b.as_ptr() as _ },
-            c: &Tile {
-                ptr: c.as_ptr() as _,
-                row_byte_stride: std::mem::size_of::<TC>() as isize,
-                col_byte_stride: (std::mem::size_of::<TC>() * K::mr()) as isize,
-                item_size: std::mem::size_of::<TC>(),
+        let mut c: Vec<TC> = vec![TC::zero(); K::mr() * K::nr()];
+        let tile = mmm_stride_storage(&mut c, 1, 0);
+        let b_store = InputStoreKer::Packed(PackedStoreKer { ptr: b.as_ptr() as _ });
+        let non_linear_ops = tvec!(
+            FusedKerSpec::Clear,
+            FusedKerSpec::AddMatMul {
+                pa: unsafe { pa.as_ptr_unchecked::<u8>() as _ },
+                pb: &b_store,
+                k,
+                cpu_variant: 0,
             },
-            linear: &LinearSpec::k(k),
-            non_linear: std::ptr::null(),
-        });
+            FusedKerSpec::Store(tile),
+            FusedKerSpec::Done
+        );
+        let err = K::kernel(&non_linear_ops);
         assert_eq!(err, 0);
         let expected = vec![k.as_(); K::mr()];
         assert_eq!(c[..K::mr()], expected);
