@@ -1,6 +1,7 @@
 use super::ScratchSpaceFusedNonLinear;
 use super::*;
 use crate::frame::Packer;
+use anyhow::Context;
 use num_traits::{AsPrimitive, Zero};
 use std::fmt;
 use std::fmt::Debug;
@@ -21,37 +22,28 @@ pub trait MatMatMul:
 
     fn internal_type(&self) -> DatumType;
 
-    unsafe fn a_packed(&self, item_size: usize, k: usize) -> MatrixStoreSpec;
+    unsafe fn a_packed(&self, item_size: usize, k: usize) -> PackedStoreSpec;
 
-    unsafe fn b_packed(&self, item_size: usize, k: usize) -> MatrixStoreSpec;
+    unsafe fn b_packed(&self, item_size: usize, k: usize) -> InputStoreSpec;
     unsafe fn b_from_data_and_offsets(
         &self,
         item_size: usize,
         rows_offsets: &[isize],
         cols_offsets: &[isize],
-    ) -> MatrixStoreSpec;
+    ) -> InputStoreSpec;
 
-    unsafe fn c_view(&self) -> MatrixStoreSpec;
-    unsafe fn c_view_with_axis(&self, m_axis: usize, n_axis: usize) -> MatrixStoreSpec;
+    unsafe fn c_view(&self) -> OutputStoreSpec;
+    unsafe fn c_view_with_axis(&self, m_axis: usize, n_axis: usize) -> OutputStoreSpec;
     unsafe fn c_from_data_and_strides(
         &self,
         item_size: usize,
         row_stride: isize,
         col_stride: isize,
-    ) -> MatrixStoreSpec;
+    ) -> OutputStoreSpec;
 
-    unsafe fn run(
-        &self,
-        m: usize,
-        k: usize,
-        n: usize,
-        a: &MatrixStore,
-        b: &MatrixStore,
-        c: &mut MatrixStore,
-        non_linear: &[FusedSpec],
-    ) -> anyhow::Result<()> {
+    unsafe fn run(&self, m: usize, n: usize, non_linear: &[FusedSpec]) -> anyhow::Result<()> {
         let mut scratch = self.allocate_scratch_space();
-        self.run_with_scratch_space(m, k, n, &mut *scratch, a, b, c, non_linear)
+        self.run_with_scratch_space(m, n, &mut *scratch, non_linear)
     }
 
     unsafe fn allocate_scratch_space(&self) -> Box<dyn ScratchSpace>;
@@ -59,12 +51,15 @@ pub trait MatMatMul:
     unsafe fn run_with_scratch_space(
         &self,
         m: usize,
-        k: usize,
         n: usize,
         scratch: &mut dyn ScratchSpace,
-        a: &MatrixStore,
-        b: &MatrixStore,
-        c: &mut MatrixStore,
+        non_linear: &[FusedSpec],
+    ) -> anyhow::Result<()>;
+
+    unsafe fn run_with_scratch_space_vec(
+        &self,
+        m: usize,
+        scratch: &mut dyn ScratchSpace,
         non_linear: &[FusedSpec],
     ) -> anyhow::Result<()>;
 }
@@ -125,19 +120,6 @@ where
     pub fn new() -> MatMatMulImpl<K, TI> {
         MatMatMulImpl { prefetch: crate::ops().prefetch, phantom: PhantomData }
     }
-
-    #[inline]
-    fn prefetch(&self, a: &PanelStore, b: &PanelStore) {
-        if let Some(prefetch) = self.prefetch {
-            if let PanelStore::Packed { ptr } = a {
-                prefetch(*ptr as *const u8, 512);
-            }
-            match b {
-                PanelStore::Packed { ptr } => prefetch(*ptr as *const u8, 512),
-                _ => (),
-            }
-        }
-    }
 }
 
 impl<K, TI> MatMatMul for MatMatMulImpl<K, TI>
@@ -171,12 +153,13 @@ where
         TI::datum_type()
     }
 
-    unsafe fn a_packed(&self, item_size: usize, k: usize) -> MatrixStoreSpec {
-        MatrixStoreSpec::Packed { panel_bytes: (k * K::mr() * item_size) }
+    unsafe fn a_packed(&self, item_size: usize, k: usize) -> PackedStoreSpec {
+        PackedStoreSpec { panel_bytes: (k * K::mr() * item_size) }
     }
 
-    unsafe fn b_packed(&self, item_size: usize, k: usize) -> MatrixStoreSpec {
-        MatrixStoreSpec::Packed { panel_bytes: (k * K::nr() * item_size) }
+    unsafe fn b_packed(&self, item_size: usize, k: usize) -> InputStoreSpec {
+        let panel_bytes = k * K::nr() * item_size;
+        InputStoreSpec::Packed(PackedStoreSpec { panel_bytes })
     }
 
     unsafe fn b_from_data_and_offsets(
@@ -184,7 +167,7 @@ where
         item_size: usize,
         rows_offsets: &[isize],
         cols_offsets: &[isize],
-    ) -> MatrixStoreSpec {
+    ) -> InputStoreSpec {
         debug_assert!(rows_offsets.len() > 0);
         debug_assert!(cols_offsets.len() > 0);
         // repeat the last offset to get to the panel boundary (pad to next multiple of nr)
@@ -205,15 +188,15 @@ where
         for i in 0..4 {
             *row_byte_offsets.get_unchecked_mut(rows_offsets.len() + i) = pad;
         }
-        MatrixStoreSpec::OffsetsAndPtrs { col_byte_offsets, row_byte_offsets, nr: K::nr() }
+        InputStoreSpec::OffsetsAndPtrs { col_byte_offsets, row_byte_offsets, nr: K::nr() }
     }
 
-    unsafe fn c_view(&self) -> MatrixStoreSpec {
-        MatrixStoreSpec::View { axes: None, mr: K::mr(), nr: K::nr() }
+    unsafe fn c_view(&self) -> OutputStoreSpec {
+        OutputStoreSpec::View { axes: None, mr: K::mr(), nr: K::nr() }
     }
 
-    unsafe fn c_view_with_axis(&self, m_axis: usize, n_axis: usize) -> MatrixStoreSpec {
-        MatrixStoreSpec::View { axes: Some((m_axis, n_axis)), mr: K::mr(), nr: K::nr() }
+    unsafe fn c_view_with_axis(&self, m_axis: usize, n_axis: usize) -> OutputStoreSpec {
+        OutputStoreSpec::View { axes: Some((m_axis, n_axis)), mr: K::mr(), nr: K::nr() }
     }
 
     unsafe fn c_from_data_and_strides(
@@ -221,8 +204,8 @@ where
         item_size: usize,
         row_stride: isize,
         col_stride: isize,
-    ) -> MatrixStoreSpec {
-        MatrixStoreSpec::Strides {
+    ) -> OutputStoreSpec {
+        OutputStoreSpec::Strides {
             row_byte_stride: row_stride * item_size as isize,
             col_byte_stride: col_stride * item_size as isize,
             row_item_stride: row_stride,
@@ -240,124 +223,72 @@ where
         scratch.downcast_ref::<ScratchSpaceFusedNonLinear<TI>>().is_some()
     }
 
-    unsafe fn run_with_scratch_space(
+    unsafe fn run_with_scratch_space_vec(
         &self,
         m: usize,
-        k: usize,
-        n: usize,
         scratch: &mut dyn ScratchSpace,
-        a: &MatrixStore,
-        b: &MatrixStore,
-        c: &mut MatrixStore,
         non_linear: &[FusedSpec],
     ) -> anyhow::Result<()> {
-        use anyhow::Context;
         let mr = K::mr();
-        let nr = K::nr();
         let scratch = scratch
             .downcast_mut::<ScratchSpaceFusedNonLinear<TI>>()
             .context("Wrong scratch space type")?;
-        let ref linear = LinearSpec::k(k);
+        scratch.prepare::<K>(non_linear);
         for ia in 0..m / mr {
-            let ref a = a.panel_a(ia);
-            if K::nr() == 1 && n == 1 {
-                let ref b = b.panel_b(0);
-                self.prefetch(a, b);
-                scratch.clear();
-                let non_linear = scratch.for_tile::<K>(&non_linear, ia, 0, c);
-                let ref direct_c = c.tile_c(ia, 0);
-                let err = K::kernel(&MatMatMulKerSpec {
-                    a: a as _,
-                    b: b as _,
-                    c: direct_c as _,
-                    linear,
-                    non_linear,
-                });
+            scratch.for_valid_tile::<K>(&non_linear, ia, 0);
+            let err = K::kernel(&scratch.uspecs());
+            debug_assert_eq!(err, 0, "Kernel return error {}", err);
+        }
+        if m % mr != 0 {
+            scratch.for_border_tile::<K>(&non_linear, m / mr, 0);
+            let err = K::kernel(&scratch.uspecs());
+            debug_assert_eq!(err, 0, "Kernel return error {}", err);
+            scratch.postprocess_tile::<K>(&non_linear, m / mr, 0, m % mr, 1);
+        }
+        Ok(())
+    }
+
+    unsafe fn run_with_scratch_space(
+        &self,
+        m: usize,
+        n: usize,
+        scratch: &mut dyn ScratchSpace,
+        non_linear: &[FusedSpec],
+    ) -> anyhow::Result<()> {
+        let mr = K::mr();
+        let nr = K::nr();
+        if n == 1 && K::nr() == 1 {
+            return self.run_with_scratch_space_vec(m, scratch, &non_linear);
+        }
+        let scratch = scratch
+            .downcast_mut::<ScratchSpaceFusedNonLinear<TI>>()
+            .context("Wrong scratch space type")?;
+        scratch.prepare::<K>(non_linear);
+        for ia in 0..m / mr {
+            for ib in 0..n / nr {
+                scratch.for_valid_tile::<K>(&non_linear, ia, ib);
+                let err = K::kernel(&scratch.uspecs());
                 debug_assert_eq!(err, 0, "Kernel return error {}", err);
-            } else {
-                for ib in 0..n / nr {
-                    let ref b = b.panel_b(ib);
-                    self.prefetch(a, b);
-                    scratch.clear();
-                    let ref direct_c = c.tile_c(ia, ib);
-                    let non_linear = scratch.for_tile::<K>(&non_linear, ia, ib, c);
-                    let err = K::kernel(&MatMatMulKerSpec {
-                        a: a as _,
-                        b: b as _,
-                        c: direct_c as _,
-                        linear,
-                        non_linear,
-                    });
-                    debug_assert_eq!(err, 0, "Kernel return error {}", err);
-                }
-                if n % nr != 0 {
-                    let ref b = b.panel_b(n / nr);
-                    self.prefetch(a, b);
-                    scratch.clear();
-                    let tmpc = scratch.tmp_tile_c(c.item_size(), mr, nr);
-                    let non_linear = scratch.for_tile::<K>(&non_linear, ia, n / nr, c);
-                    let err = K::kernel(&MatMatMulKerSpec {
-                        a: a as _,
-                        b: b as _,
-                        c: &tmpc,
-                        linear,
-                        non_linear,
-                    });
-                    debug_assert_eq!(err, 0, "Kernel return error {}", err);
-                    c.set_from_tile(ia, n / nr, mr, n % nr, &tmpc);
-                }
+            }
+            if n % nr != 0 {
+                scratch.for_border_tile::<K>(&non_linear, ia, n / nr);
+                let err = K::kernel(&scratch.uspecs());
+                debug_assert_eq!(err, 0, "Kernel return error {}", err);
+                scratch.postprocess_tile::<K>(&non_linear, ia, n / nr, mr, n % nr);
             }
         }
         if m % mr != 0 {
-            let ref panel_a = a.panel_a(m / mr);
-            if K::nr() == 1 && n == 1 {
-                let ref b = b.panel_b(0);
-                self.prefetch(panel_a, b);
-                scratch.clear();
-                let tmpc = scratch.tmp_tile_c(c.item_size(), mr, nr);
-                let non_linear = scratch.for_tile::<K>(&non_linear, m / mr, 0, c);
-                let err = K::kernel(&MatMatMulKerSpec {
-                    a: panel_a as _,
-                    b: b as _,
-                    c: &tmpc,
-                    linear,
-                    non_linear,
-                });
+            for ib in 0..n / nr {
+                scratch.for_border_tile::<K>(&non_linear, m / mr, ib);
+                let err = K::kernel(&scratch.uspecs());
                 debug_assert_eq!(err, 0, "Kernel return error {}", err);
-                c.set_from_tile(m / mr, 0, m % mr, nr, &tmpc);
-            } else {
-                for ib in 0..n / nr {
-                    let ref b = b.panel_b(ib);
-                    self.prefetch(panel_a, b);
-                    scratch.clear();
-                    let tmpc = scratch.tmp_tile_c(c.item_size(), mr, nr);
-                    let non_linear = scratch.for_tile::<K>(&non_linear, m / mr, ib, c);
-                    let err = K::kernel(&MatMatMulKerSpec {
-                        a: panel_a as _,
-                        b: b as _,
-                        c: &tmpc,
-                        linear,
-                        non_linear,
-                    });
-                    debug_assert_eq!(err, 0, "Kernel return error {}", err);
-                    c.set_from_tile(m / mr, ib, m % mr, nr, &tmpc);
-                }
-                if n % nr != 0 {
-                    let ref b = b.panel_b(n / nr);
-                    self.prefetch(panel_a, b);
-                    scratch.clear();
-                    let tmpc = scratch.tmp_tile_c(c.item_size(), mr, nr);
-                    let non_linear = scratch.for_tile::<K>(&non_linear, m / mr, n / nr, c);
-                    let err = K::kernel(&MatMatMulKerSpec {
-                        a: panel_a as _,
-                        b: b as _,
-                        c: &tmpc,
-                        linear,
-                        non_linear,
-                    });
-                    debug_assert_eq!(err, 0, "Kernel return error {}", err);
-                    c.set_from_tile(m / mr, n / nr, m % mr, n % nr, &tmpc);
-                }
+                scratch.postprocess_tile::<K>(&non_linear, m / mr, ib, m % mr, nr);
+            }
+            if n % nr != 0 {
+                scratch.for_border_tile::<K>(&non_linear, m / mr, n / nr);
+                let err = K::kernel(&scratch.uspecs());
+                debug_assert_eq!(err, 0, "Kernel return error {}", err);
+                scratch.postprocess_tile::<K>(&non_linear, m / mr, n / nr, m % mr, n % nr);
             }
         }
         Ok(())
