@@ -299,14 +299,18 @@ where
 
         op.run(
             m,
-            k,
             n,
-            &op.a_packed(TA::datum_type().size_of(), k).wrap(&packed_a.view()),
-            &op.b_packed(TB::datum_type().size_of(), k).wrap(&packed_b.view()),
-            &mut op
-                .c_from_data_and_strides(TC::datum_type().size_of(), n as isize, 1)
-                .wrap(&found.view_mut()),
-            &[],
+            &[
+                FusedSpec::AddMatMul {
+                    a: op.a_packed(TA::datum_type().size_of(), k).wrap(&packed_a.view()),
+                    b: op.b_packed(TB::datum_type().size_of(), k).wrap(&packed_b.view()),
+                    k,
+                },
+                FusedSpec::Store(
+                    op.c_from_data_and_strides(TC::datum_type().size_of(), n as isize, 1)
+                        .wrap(&found.view_mut()),
+                ),
+            ],
         )
         .unwrap();
 
@@ -355,14 +359,16 @@ where
 
         let mut c = Tensor::uninitialized::<TC>(&[m, 1]).unwrap();
 
+        let pa = op.a_packed(TA::datum_type().size_of(), k).wrap(&packed_a.view());
+        let pb = op.b_packed(b.datum_type().size_of(), k).wrap(&packed_b.view());
+
         op.run(
             m,
-            k,
             1,
-            &op.a_packed(TA::datum_type().size_of(), k).wrap(&packed_a.view()),
-            &op.b_packed(b.datum_type().size_of(), k).wrap(&packed_b.view()),
-            &mut op.c_view().wrap(&c.view_mut()),
-            &[],
+            &[
+                FusedSpec::AddMatMul { k, a: pa, b: pb },
+                FusedSpec::Store(op.c_view().wrap(&c.view_mut())),
+            ],
         )
         .unwrap();
 
@@ -412,20 +418,16 @@ where
     op.b_pack(k).pack(packed_b.view_mut(), b.view(), 0, 1);
 
     let mut found = Tensor::zero::<TC>(&[m, n]).unwrap();
+    let c_store = op
+        .c_from_data_and_strides(TC::datum_type().size_of(), n as isize, 1)
+        .wrap(&mut found.view_mut());
+    let mut spec: TVec<FusedSpec> = spec.into();
+    let a_store = op.a_packed(TA::datum_type().size_of(), k).wrap(&packed_a.view());
+    let b_store = op.b_packed(TB::datum_type().size_of(), k).wrap(&packed_b.view());
+    spec.insert(0, FusedSpec::AddMatMul { k, a: a_store, b: b_store });
+    spec.push(FusedSpec::Store(c_store));
 
-    op.run(
-        m,
-        k,
-        n,
-        &op.a_packed(TA::datum_type().size_of(), k).wrap(&packed_a.view()),
-        &op.b_packed(TB::datum_type().size_of(), k).wrap(&packed_b.view()),
-        &mut op
-            .c_from_data_and_strides(TC::datum_type().size_of(), n as isize, 1)
-            .wrap(&mut found.view_mut()),
-        spec,
-    )
-    .unwrap();
-
+    op.run(m, n, &spec).unwrap();
     let mut inter = Tensor::zero::<TI>(&[m, n]).unwrap();
     for x in 0..n {
         for y in 0..m {
@@ -457,13 +459,19 @@ where
     usize: AsPrimitive<TI>,
 {
     let bias = (0..m).map(|i| i.as_()).collect::<Vec<TI>>();
-    fused_op::<K, TA, TB, TC, TI, _>(m, k, n, &[FusedSpec::PerRowAdd(&tensor1(&*bias))], |exp| {
-        for x in 0..n {
-            for y in 0..m {
-                exp[x + y * n] += bias[y]
+    fused_op::<K, TA, TB, TC, TI, _>(
+        m,
+        k,
+        n,
+        &[FusedSpec::BinPerRow(&tensor1(&*bias), BinOp::Add)],
+        |exp| {
+            for x in 0..n {
+                for y in 0..m {
+                    exp[x + y * n] += bias[y]
+                }
             }
-        }
-    })
+        },
+    )
 }
 
 pub unsafe fn row_mul<K: MatMatMulKer<TI> + 'static, TA, TB, TC, TI>(
@@ -480,13 +488,19 @@ where
     usize: AsPrimitive<TI>,
 {
     let bias = (0..m).map(|i| i.as_()).collect::<Vec<TI>>();
-    fused_op::<K, TA, TB, TC, TI, _>(m, k, n, &[FusedSpec::PerRowMul(&tensor1(&*bias))], |exp| {
-        for x in 0..n {
-            for y in 0..m {
-                exp[x + y * n] *= bias[y]
+    fused_op::<K, TA, TB, TC, TI, _>(
+        m,
+        k,
+        n,
+        &[FusedSpec::BinPerRow(&tensor1(&*bias), BinOp::Mul)],
+        |exp| {
+            for x in 0..n {
+                for y in 0..m {
+                    exp[x + y * n] *= bias[y]
+                }
             }
-        }
-    })
+        },
+    )
 }
 
 pub unsafe fn col_add<K: MatMatMulKer<TI> + 'static, TA, TB, TC, TI>(
@@ -503,7 +517,7 @@ where
     usize: AsPrimitive<TI>,
 {
     let bias = (0..n).map(|i| i.as_()).collect::<Vec<TI>>();
-    fused_op::<K, TA, TB, TC, TI, _>(m, k, n, &[FusedSpec::PerColAdd(&tensor1(&*bias))], |exp| {
+    fused_op::<K, TA, TB, TC, TI, _>(m, k, n, &[FusedSpec::BinPerCol(&tensor1(&*bias), BinOp::Add)], |exp| {
         for x in 0..n {
             for y in 0..m {
                 exp[x + y * n] += bias[x]
@@ -526,7 +540,7 @@ where
     usize: AsPrimitive<TI>,
 {
     let bias = (0..n).map(|i| i.as_()).collect::<Vec<TI>>();
-    fused_op::<K, TA, TB, TC, TI, _>(m, k, n, &[FusedSpec::PerColMul(&tensor1(&*bias))], |exp| {
+    fused_op::<K, TA, TB, TC, TI, _>(m, k, n, &[FusedSpec::BinPerCol(&tensor1(&*bias), BinOp::Mul)], |exp| {
         for x in 0..n {
             for y in 0..m {
                 exp[x + y * n] *= bias[x]
@@ -550,14 +564,21 @@ where
 {
     let d = (0..m * n).map(|i| i.as_()).collect::<Vec<TI>>();
     let d = tensor1(&*d).into_shape(&[m, n]).unwrap();
-    fused_op::<K, TA, TB, TC, TI, _>(m, k, n, &[FusedSpec::AddUnicast(d.view())], |exp| {
-        for x in 0..n {
-            for y in 0..m {
-                exp[x + y * n] =
-                    (exp[x + y * n] + d.as_slice::<TI>().unwrap()[x + n * y]).as_().as_();
+    let store_spec = OutputStoreSpec::View { axes: None, mr: K::mr(), nr: K::nr() };
+    fused_op::<K, TA, TB, TC, TI, _>(
+        m,
+        k,
+        n,
+        &[FusedSpec::AddUnicast(store_spec.wrap(&d.view()))],
+        |exp| {
+            for x in 0..n {
+                for y in 0..m {
+                    exp[x + y * n] =
+                        (exp[x + y * n] + d.as_slice::<TI>().unwrap()[x + n * y]).as_().as_();
+                }
             }
-        }
-    })
+        },
+    )
 }
 
 pub unsafe fn max<K: MatMatMulKer<TI>, TA, TB, TC, TI>(
@@ -574,9 +595,13 @@ where
     usize: AsPrimitive<TI>,
 {
     let five: TI = 5.as_();
-    fused_op::<K, TA, TB, TC, TI, _>(m, k, n, &[FusedSpec::Max(&tensor0(five))], |exp| {
-        exp.iter_mut().for_each(|x| *x = if *x < five { five } else { *x })
-    })
+    fused_op::<K, TA, TB, TC, TI, _>(
+        m,
+        k,
+        n,
+        &[FusedSpec::BinScalar(&tensor0(five), BinOp::Max)],
+        |exp| exp.iter_mut().for_each(|x| *x = if *x < five { five } else { *x }),
+    )
 }
 
 pub unsafe fn min<K: MatMatMulKer<TI>, TA, TB, TC, TI>(
@@ -593,9 +618,13 @@ where
     usize: AsPrimitive<TI>,
 {
     let five: TI = 5.as_();
-    fused_op::<K, TA, TB, TC, TI, _>(m, k, n, &[FusedSpec::Min(&tensor0(five))], |exp| {
-        exp.iter_mut().for_each(|x| *x = if *x > five { five } else { *x })
-    })
+    fused_op::<K, TA, TB, TC, TI, _>(
+        m,
+        k,
+        n,
+        &[FusedSpec::BinScalar(&tensor0(five), BinOp::Min)],
+        |exp| exp.iter_mut().for_each(|x| *x = if *x > five { five } else { *x }),
+    )
 }
 
 #[derive(Clone, Debug)]
@@ -692,19 +721,28 @@ impl<TA: LADatum, TB: LADatum> ConvProblem<TA, TB> {
                 .unwrap();
             op.run(
                 self.m(),
-                self.k(),
                 self.n(),
-                &op.a_packed(TA::datum_type().size_of(), self.k()).wrap(&packed_a.view()),
-                &op.b_from_data_and_offsets(
-                    TB::datum_type().size_of(),
-                    &self.data_rows_offsets(),
-                    &self.data_cols_offsets(),
-                )
-                .wrap(&self.data.view()),
-                &mut op
-                    .c_from_data_and_strides(TC::datum_type().size_of(), self.n() as isize, 1)
-                    .wrap(&found.view_mut()),
-                &[],
+                &[
+                    FusedSpec::AddMatMul {
+                        a: op.a_packed(TA::datum_type().size_of(), self.k()).wrap(&packed_a.view()),
+                        b: op
+                            .b_from_data_and_offsets(
+                                TB::datum_type().size_of(),
+                                &self.data_rows_offsets(),
+                                &self.data_cols_offsets(),
+                            )
+                            .wrap(&self.data.view()),
+                        k: self.k(),
+                    },
+                    FusedSpec::Store(
+                        op.c_from_data_and_strides(
+                            TC::datum_type().size_of(),
+                            self.n() as isize,
+                            1,
+                        )
+                        .wrap(&found.view_mut()),
+                    ),
+                ],
             )
             .unwrap();
             found
